"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = loader;
exports.absPackageName = absPackageName;

var babylon = _interopRequireWildcard(require("babylon"));

var _babelTraverse = _interopRequireDefault(require("babel-traverse"));

var _babelGenerator = _interopRequireDefault(require("babel-generator"));

var t = _interopRequireWildcard(require("babel-types"));

var _babelTemplate = _interopRequireDefault(require("babel-template"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var buildDynamicImport = (0, _babelTemplate.default)("\n  new Promise((resolve) => {\n    require.ensure([], (require) => {\n      resolve(require(PACKAGE_NAME));\n    });\n  })\n");
var buildRequire = (0, _babelTemplate.default)("\n  require(DEF_FILE).default ? require(DEF_FILE).default : require(DEF_FILE)\n");

function loader(source) {
  var ast = babylon.parse(source);
  var that = this;
  (0, _babelTraverse.default)(ast, {
    CallExpression: function CallExpression(path) {
      var typeNode, propsNode, type, packageName;

      if (path.node.arguments.length < 2) {
        return;
      }

      typeNode = path.get('arguments.0');
      propsNode = path.get('arguments.1');

      if (typeNode.isStringLiteral() && propsNode.isNullLiteral()) {
        type = typeNode.node.value;
        packageName = _utils.componentMap.get('type');

        if (!packageName) {
          return;
        }

        packageName = absPackageName(packageName, that);
        path.get('arguments.1').replaceWith(t.objectExpression([t.objectProperty(t.stringLiteral('packageName'), t.stringLiteral(packageName)), t.objectProperty(t.stringLiteral('loader'), buildLoader(packageName)), t.objectProperty(t.stringLiteral('builder'), buildBuilder(packageName))].concat(_toConsumableArray(builderCannerConfig(packageName, type)))));
      }

      if (typeNode.isStringLiteral() && propsNode.isObjectExpression()) {
        type = typeNode.node.value;
        var properties = getProperties(propsNode);

        if ('packageName' in properties) {
          packageName = properties.packageName;
        } else {
          packageName = _utils.componentMap.get(type, properties.ui);
        }

        if (!packageName) {
          return;
        }

        packageName = absPackageName(packageName, that);
        propsNode.node.properties.push(t.objectProperty(t.stringLiteral('packageName'), t.stringLiteral(packageName)));
        propsNode.node.properties.push(t.objectProperty(t.stringLiteral('loader'), buildLoader(packageName)));
        propsNode.node.properties.push(t.objectProperty(t.stringLiteral('builder'), buildBuilder(packageName)));
        propsNode.node.properties = propsNode.node.properties.concat(builderCannerConfig(packageName, type));
      }
    }
  });
  return (0, _babelGenerator.default)(ast, {}, source).code;
}

function getProperties(propsNode) {
  return propsNode.node.properties.reduce(function (result, property) {
    return _objectSpread({}, result, _defineProperty({}, getPropertyKey(property), getPropertyValue(property)));
  }, {});
}

function getPropertyKey(property) {
  return property.key.name;
}

function getPropertyValue(property) {
  if (t.isStringLiteral(property.value)) {
    return property.value.value;
  } // for now, we only need ui and packagename, so If it's not string literal we can ignore it.


  return undefined;
}

function absPackageName(sourcePkgName, context) {
  var packageName = sourcePkgName;

  if (packageName.startsWith('.') || packageName.startsWith(_path.default.sep)) {
    // customize component
    packageName = _path.default.resolve(context.context, sourcePkgName);
  } else {
    // deployed component
    var paths = require.resolve(packageName).split(_path.default.sep);

    var dirPathIndex = paths.indexOf(packageName);
    packageName = paths.slice(0, dirPathIndex - 1).join(_path.default.sep);
  }

  return packageName;
}

function buildLoader(packageName) {
  return buildDynamicImport({
    PACKAGE_NAME: t.stringLiteral(packageName)
  }).expression;
}

function buildBuilder(packageName) {
  var builder;
  var defFile = "".concat(packageName, "/canner.def.js");

  if (_fs.default.existsSync(defFile)) {
    // to check defFile is exist or not, if not this line will throw an error
    builder = buildRequire({
      DEF_FILE: t.stringLiteral(defFile)
    }).expression;
  } else {
    builder = t.nullLiteral();
  }

  return builder;
}

function builderCannerConfig(packageName, type) {
  var canner = {},
      properties = [];

  try {
    canner = require("".concat(packageName, "/package.json")).canner || {};
  } catch (e) {
    canner = {};
  }

  Object.keys(canner).forEach(function (key) {
    if (key === 'cannerDataType') {
      if (type !== 'json') {
        // the component of json type is same as object, but the type can't be changed
        properties.push(t.objectProperty(t.stringLiteral('type'), t.stringLiteral(canner[key])));
      }
    } else if (typeof canner[key] === 'string') {
      properties.push(t.objectProperty(t.stringLiteral(key), t.stringLiteral(canner[key])));
    } else if (typeof canner[key] === 'number') {
      properties.push(t.objectProperty(t.stringLiteral(key), t.numberLiteral(canner[key])));
    } else if (typeof canner[key] === 'boolean') {
      properties.push(t.objectProperty(t.stringLiteral(key), t.booleanLiteral(canner[key])));
    } else {
      throw new Error("Not support the type of object and array config in canner yet, but got key: ".concat(key, " with value: ").concat(canner[key], "."));
    }
  });
  return properties;
}